'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.parseBindingDefinition = parseBindingDefinition;
exports.createMapScopeToProps = createMapScopeToProps;
exports.wrapWithDecorators = wrapWithDecorators;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseBindingDefinition(bindingDefinition) {
    var match = bindingDefinition.match(/^([@=<&])(\??)/);

    if (!match) {
        throw new Error('Defintion of binding \'' + bindingDefinition + '\' is not a valid');
    }

    var binding = match[1];
    var optional = match[2] === '?';

    // We only support '<' and '&'
    if (binding !== '<' && binding !== '&') {
        throw new Error('Binding \'' + binding + '\' is not a supported binding');
    }

    return { binding: binding, optional: optional };
}

function createMapScopeToProps(bindings) {
    // We compile the details as early as possible because we want to validate the bindings before using the bindings.
    var bindingDetails = Object.keys(bindings).reduce(function (details, name) {
        details[name] = parseBindingDefinition(bindings[name]);

        return details;
    }, {});

    return function mapScopeToProps(scope) {
        return Object.keys(bindingDetails).reduce(function (props, propName) {
            var _bindingDetails$propN = bindingDetails[propName],
                binding = _bindingDetails$propN.binding,
                optional = _bindingDetails$propN.optional;

            // If the binding is optional and no value is passed we don't want to add it to the props

            if (optional && typeof scope[propName] === 'undefined') {
                return props;
            }

            // We expect that properties which are bound with & are functions
            if (binding === '&') {
                props[propName] = function (event) {
                    return scope[propName]({ $event: event });
                };
            } else {
                props[propName] = scope[propName];
            }

            return props;
        }, {});
    };
}

function wrapWithDecorators(decorators) {
    var compose = function compose(render, decorate) {
        // We create a new render function which takes props from outside
        var renderDecorator = function renderDecorator(props) {
            // Function which takes the passed props to render the component
            var renderComponentWithProps = function renderComponentWithProps() {
                return render(props);
            };

            // Pass the render function for component rendering to the decorate function
            // as render function
            return decorate(renderComponentWithProps);
        };

        return renderDecorator;
    };

    return function withDecorators(Component) {
        var render = function render(props) {
            return _react2.default.createElement(Component, props);
        };

        return decorators.reduce(compose, render);
    };
}